function [RES]=DS_CASEC_KSMIRNOV(Dat,ALPHA,Sizesample)
% clc
% clear variables
% close
%% Solve The NAFEMS CHALLENGE PROBLEM CASE C: 10 sampled points for each parameter of interest (R,L,C)
% It has been used the Kolmogrov-Smirnov Test and build PBOX based on the 10 samples population
% And Compute Reliability w.r.t. 3 requirements
%1) Vc(10ms)>0.9
%2) Vc(t=tr)>0.9 and tr<8ms
%3) Z>=1
%% DATA
% Rs=[861,87,430,798,219,152,64,361,224,614];
% Ls=[4.1,8.8,4.0,7.6,0.7,3.9,7.1,5.9,8.2,5.1];
% Cs=[9,5.2,3.8,4.9,2.9,8.3,7.7,5.8,10,0.7];
% Ls=Ls*10^-3;Cs=Cs*10^-6;
%% INPUTS

%Sizesample=10^2;% Samples for the min-max search within parameter boxes
%ALPHA=0.1; % Critcal value for hte KS test (1-Required Confidence Level)
G=3; % G are the number of standard deviation ASSUMED for PBOX right tails truncation (otherwiseinfinite tails) for the left nonegativity 

Rs=Dat.CASEC_Rs;Ls=Dat.CASEC_Ls;Cs=Dat.CASEC_Cs ;
Dat.Rs=Rs;Dat.Ls=Ls;Dat.Cs=Cs;
%% KS TEST 
% Different alpha led to differen critical value, hence more narrow or broader cdf limits
% KS test to obtain the Critical values (is a table for small sample size)
[~,~,~,cr] = kstest(Dat.CASEC_Rs,'alpha',ALPHA);
[~,~,~,cl] = kstest(Dat.CASEC_Ls,'alpha',ALPHA);
[~,~,~,cc] = kstest(Dat.CASEC_Cs,'alpha',ALPHA);
%% DEFINE UPPER TRUNCATION BOUNDs (sample mean +G times the sample standard deviation)
MAXRs=mean(Rs)+G*std(Rs);
MAXLs=mean(Ls)+G*std(Ls);
MAXCs=mean(Cs)+G*std(Cs);
%% COMPUTE CDFs Steps
F=0:1/max(size(Rs)):1;  %discrete CDF for the parameter(step 1/10 for each param sample) F=[F 1] to account long tail....then fix MAX
F=[F 1];
% bound ecdf R F of R(10 samples) upper and lower
FRu=F+cr; % compute upper bound(on probabilities) using critical value from Kolmogorov smironov test
FRl=F-cr; % compute lower bound(on probabilities) using critical value from Kolmogorov smironov test
FRu(FRu>1)=1;  % fix upper probability (CDF<=1)
FRl(FRl<0)=0;  % fix lower buonds on probability (CDF<=1)
FRl(end)=1;    % fix lower buonds on probability
%% L
FRl(end)=1;
F=0:1/max(size(Ls)):1;  %discrete CDF for the parameter(step 1/10 for each param sample) F=[F 1] to account logn tail....then fix MAX
F=[F 1];
FLu=F+cl; % compute upper bound(on probabilities) using critical value from Kolmogorov smironov test
FLl=F-cl; % compute lower bound(on probabilities) using critical value from Kolmogorov smironov test
FLu(FLu>1)=1;  % fix upper probability (CDF<=1)
FLl(FLl<0)=0;  % fix lower buonds on probability (CDF<=1)
FLl(end)=1;    % fix lower buonds on probability
%% R
F=0:1/max(size(Cs)):1;  %discrete CDF for the C samples
F=[F 1];  % one final slot for the fixed maximum value
% bound ecdf C F of C(10 samples) upper and lower
FCu=F+cc;% compute upper bound (on probabilities) using critical value from Kolmogorov smironov test
FCl=F-cc;% compute lower bound (on probabilities) using critical value from Kolmogorov smironov test
FCu(FCu>1)=1;% adjust upper bound (CDF<=1)
FCl(FCl<0)=0; % adjust lower bound (CDF<=1)
FCl(end)=1;
%% P-BOXES {LowerB, UpperB, Mass} for all parameter R,L,C
PBR=[[FRl;zeros(1,max(size(FRl)))],[FRu;ones(1,max(size(FRu)))];[0 sort(Dat.Rs) MAXRs 0 sort(Dat.Rs) MAXRs]]; %R
PBL=[[FLl;zeros(1,length(FLl))],[FLu;ones(1,length(FLu))];[0 sort(Dat.Ls) MAXLs 0 sort(Dat.Ls) MAXLs]];       %L
PBC=[[FCl;zeros(1,length(FCl))],[FCu;ones(1,length(FCu))];[0 sort(Dat.Cs) MAXCs 0 sort(Dat.Cs) MAXCs]];       %C
%% TRANSLATE to DS STRUCTURE ( % discretization % )
DSR=PBOX2DS(PBR);
DSL=PBOX2DS(PBL);
DSC=PBOX2DS(PBC);
a=DSR(:,1);
b=DSR(:,2);
abR=DSR(:,3);
c=DSL(:,1);
d=DSL(:,2);
cdL=DSL(:,3);
e=DSC(:,1);
f=DSC(:,2);
efC=DSC(:,3);
TOTALCELLS=length(a)*length(c)*length(e);
VC10MAX=zeros(1,TOTALCELLS);VC10MIN=zeros(1,TOTALCELLS);VC08MAX=zeros(1,TOTALCELLS); VC08MIN=zeros(1,TOTALCELLS); ZMAX=zeros(1,TOTALCELLS); ZMIN=zeros(1,TOTALCELLS); Mass=zeros(TOTALCELLS,1);
IDX=0;
for k=1:size(a)
    for s=1:size(c)
        for j=1:size(e)
            tic
            IDX= IDX+1;
            %% Compute mass of the focal element in output
            Mass(IDX,1)=abR(k)*cdL(s)*efC(j);
            % set the focal cell
            A=a(k);B=b(k);C=c(s);D=d(s);E=e(j);F=f(j);
            % latin hypercube design
            X = lhsdesign(Sizesample,3);
            % compute parameter
            R = A + X(:,1).*(B-A);
            L = C + X(:,2).*(D-C);
            C = E + X(:,3).*(F-E);
            % compute damping factor
            Zeta=0.5*R.*((C./L).^0.5);
            % eliminate underdumped samples
            R(Zeta<1)=[];
            L(Zeta<1)=[];
            C(Zeta<1)=[];
            %  Zeta(Zeta<1)=[];
            %%  System Solver
            S1=sqrt((R./(2*L)).^2-1./(L.*C))-R./(2*L);
            S2=-sqrt((R./(2*L)).^2-1./(L.*C))-R./(2*L);
            A1=S2./(S1-S2);
            A2=S1./(S2-S1);
            %t=0:0.001:0.021;
            %Vc=repmat(A1,1,length(t)).*exp(S1*t)+repmat(A2,1,length(t)).*exp(S2*t)+1;
            % RES.Vc=Vc;
            Vc10=repmat(A1,1,length(0.01)).*exp(S1*0.01)+repmat(A2,1,length(0.01)).*exp(S2*0.01)+1;
            Vc08=repmat(A1,1,length(0.008)).*exp(S1*0.008)+repmat(A2,1,length(0.008)).*exp(S2*0.08)+1;
            RES.Vc10=Vc10;
            %% Save Min-Max of the Variable in exam
            VC10MIN(IDX)=min(Vc10);
            VC10MAX(IDX)=max(Vc10);
            VC08MIN(IDX)=min(Vc08);
            VC08MAX(IDX)=max(Vc08);
            ZMAX(IDX)= max(Zeta);
            ZMIN(IDX)= min(Zeta);
            %  ECDF Vc(8ms)[F(VC08=0.9)=probability of requirement 2 not sattisfied]
            % figure(1)
            %  hold on
            % ecdf(RES.Vc08)
            % %  ECDF Vc(10ms)[F(VC10=0.9)=probability of requirement 1 not sattisfied]
            % figure(2)
            %  hold on
            % ecdf(RES.Vc10)
        end
    end
end

                                                %% PLOT %% 
%% PLOT INPUT
%     figure(1)
%     STYLE={'-',':','-.'};
%     COLOR={'r','b','k'};
%     [fr,xr]=ecdf(Rs);
%     xr=[0;xr;MAXRs];
%     fr=[0;fr;1];
%     hold on
%     if Alfa==1;   stairs(xr,fr,'linestyle','-','linewidth',1,'Marker','x');end
%     stairs(xr,[FRu(1) FRu],'linestyle','--','linewidth',2,'Color',COLOR{Alfa});
%     stairs(xr,[0 FRl],'linestyle','-','linewidth',2,'Color',COLOR{Alfa});
%     xlabel('R')
%     ylabel(' ')
%     legend('ECDF(R)','Pl \alpha=0.01','Bel \alpha=0.01','Pl \alpha=0.1','Bel \alpha=0.1','Pl \alpha=0.2','Bel \alpha=0.2');
%     fig=gcf;
%     set(findall(fig,'-property','FontSize'),'FontSize',16)

%     figure(2)
%     [fr,xr]=ecdf(Ls);
%     xr=[0;xr;MAXLs];
%     fr=[0;fr;1];
%     hold on
%     if Alfa==1;  stairs(xr,fr,'linestyle','--','linewidth',1,'Marker','x');end
%     stairs(xr,[FLu(1) FLu],'linestyle','--','linewidth',2,'Color',COLOR{Alfa});
%     stairs(xr,[0 FLl],'linestyle','-','linewidth',2,'Color',COLOR{Alfa});
%     xlabel('L')
%     ylabel('Comnulative plausibility/belif functions');
%     legend('ECDF(L)','Pl \alpha=0.01','Bel \alpha=0.01','Pl \alpha=0.1','Bel \alpha=0.1','Pl \alpha=0.2','Bel \alpha=0.2');
%     fig=gcf;
%     set(findall(fig,'-property','FontSize'),'FontSize',16)
%
%     figure(3)
%     [fr,xr]=ecdf(Cs);
%     xr=[0;xr;MAXCs];
%     fr=[0;fr;1];
%     hold on
%     if Alfa==1; stairs(xr,fr,'linestyle','-','linewidth',1,'Marker','x');end
%     stairs(xr,[FCu(1) FCu],'linestyle','--','linewidth',2,'Color',COLOR{Alfa});
%     stairs(xr,[0 FCl],'linestyle','-','linewidth',2,'Color',COLOR{Alfa});
%     xlabel('C');
%     ylabel('');
%     legend('ECDF(C)','Pl \alpha=0.01','Bel \alpha=0.01','Pl \alpha=0.1','Bel \alpha=0.1','Pl \alpha=0.2','Bel \alpha=0.2');
%     fig=gcf;
%     set(findall(fig,'-property','FontSize'),'FontSize',16)
% end
% stairs([0 sort(Ls) MAXLs],FLl)
%% PLOT OUTPUT
  dsTR=dsstruct([VC08MIN',VC08MAX',Mass]);
  dsVC=dsstruct([VC10MIN',VC10MAX',Mass]);
  dsZ=dsstruct([ZMIN',ZMAX',Mass]);
% 
  RES.dsTR=dsTR;
  RES.dsVC=dsVC;
  RES.dsZ=dsZ;
% 
% down(:,1)=dsTR.ds(:,1);
% down(:,2)=dsTR.ds(:,3);
% down=sortrows(down,1);
% up(:,1)=dsTR.ds(:,2);
% up(:,2)=dsTR.ds(:,3);
% up=sortrows(up,1);
% for i=2:size(dsTR.ds,1)
%     down(i,2)=down(i-1,2)+down(i,2);
%     up(i,2)=up(i-1,2)+up(i,2);
% end
% [xa,xb]=stairs([down(1,1);down(:,1);up(size(up,1),1)],[0;down(:,2);1]);
% [xc,xd]=stairs([down(1,1);up(:,1)],[0;up(:,2)]);
% % %plot(xa,xb,'--k',xc,xd,'k','linewidth',1.5);
% figure(3)
% hold on;
% plot(xa,xb,'--k','linewidth',1.5);
% plot(xc,xd,'-k','linewidth',1.5);
% % %xlabel('x','FontSize',12,'fontweight','bold');
% % %ylabel('p([0,x])','FontSize',12,'fontweight','bold');
% set(gca,'fontsize',12)
% xlim([down(1,1)-up(size(up,1),1)*0.02,up(size(up,1),1)*1.02]);
% % % xlim([0-100000*0.02,100000]);
% ylim([-0.0,1.02]);
% % %legend('Pl([0,x])','Bel([0,x])',4);
% legend('Pl','Bel',4);
% xlabel('VC(t=08ms)');
% ylabel('Comnulative plausibility/belif functions');
% title('Pbox VC08');
% % dscdf(dsTR)
% figure(4)
% hold on
% % ecdf(ZMIN)
% % ecdf(ZMAX)
% down(:,1)=dsZ.ds(:,1);
% down(:,2)=dsZ.ds(:,3);
% down=sortrows(down,1);
% up(:,1)=dsZ.ds(:,2);
% up(:,2)=dsZ.ds(:,3);
% up=sortrows(up,1);
% for i=2:size(dsZ.ds,1)
%     down(i,2)=down(i-1,2)+down(i,2);
%     up(i,2)=up(i-1,2)+up(i,2);
% end
% [xa,xb]=stairs([down(1,1);down(:,1);up(size(up,1),1)],[0;down(:,2);1]);
% [xc,xd]=stairs([down(1,1);up(:,1)],[0;up(:,2)]);
% % %plot(xa,xb,'--k',xc,xd,'k','linewidth',1.5);
% hold on;
% plot(xa,xb,'--k','linewidth',1.5);
% plot(xc,xd,'-k','linewidth',1.5);
% % %xlabel('x','FontSize',12,'fontweight','bold');
% % %ylabel('p([0,x])','FontSize',12,'fontweight','bold');
% set(gca,'fontsize',12)
% xlim([down(1,1)-up(size(up,1),1)*0.02,up(size(up,1),1)*1.02]);
% % % xlim([0-100000*0.02,100000]);
% ylim([-0.0,1.02]);
% % %legend('Pl([0,x])','Bel([0,x])',4);
% legend('Pl','Bel',4);
% xlabel('Z');
% ylabel('Comnulative plausibility/belif functions');
% title('Pbox Z');
% 
% figure(5)
% hold on
% % ecdf(VCMIN)
% % ecdf(VCMAX)
% down(:,1)=dsVC.ds(:,1);
% down(:,2)=dsVC.ds(:,3);
% down=sortrows(down,1);
% up(:,1)=dsVC.ds(:,2);
% up(:,2)=dsVC.ds(:,3);
% up=sortrows(up,1);
% for i=2:size(dsVC.ds,1)
%     down(i,2)=down(i-1,2)+down(i,2);
%     up(i,2)=up(i-1,2)+up(i,2);
% end
% [xa,xb]=stairs([down(1,1);down(:,1);up(size(up,1),1)],[0;down(:,2);1]);
% [xc,xd]=stairs([down(1,1);up(:,1)],[0;up(:,2)]);
% % %plot(xa,xb,'--k',xc,xd,'k','linewidth',1.5);
% hold on;
% plot(xa,xb,'--k','linewidth',1.5);
% plot(xc,xd,'-k','linewidth',1.5);
% % %xlabel('x','FontSize',12,'fontweight','bold');
% % %ylabel('p([0,x])','FontSize',12,'fontweight','bold');
% set(gca,'fontsize',12)
% xlim([down(1,1)-up(size(up,1),1)*0.02,up(size(up,1),1)*1.02]);
% % % xlim([0-100000*0.02,100000]);
% ylim([-0.0,1.02]);
% % %legend('Pl([0,x])','Bel([0,x])',4);
% legend('Pl','Bel',4);
% xlabel('Vc(t=10ms)');
% ylabel('Comnulative plausibility/belif functions');
% title('Pbox Vc(t=10ms)');
end
